from pyvenafi.tpp.attributes._helper import IterableMeta


class Classes(metaclass=IterableMeta):
    acme_account = "ACME Account"
    acme_authorization = "ACME Authorization"
    acme_challenge = "ACME Challenge"
    acme_order = "ACME Order"
    acme_root = "ACME Root"
    aws_ec2_instance_monitor = "AWS EC2 Instance Monitor"
    activedirectory_identity_driver = "ActiveDirectory Identity Driver"
    adaptable_app = "Adaptable App"
    adaptable_bulk_app = "Adaptable Bulk App"
    adaptable_ca = "Adaptable CA"
    adaptable_credential = "Adaptable Credential"
    adaptable_credential_connector = "Adaptable Credential Connector"
    adaptable_flow_action_base = "Adaptable Flow Action Base"
    adaptable_workflow = "Adaptable Workflow"
    agent = "Agent"
    agent_base = "Agent Base"
    agent_certificate_base = "Agent Certificate Base"
    agent_certificate_discovery = "Agent Certificate Discovery"
    agent_certificate_driver = "Agent Certificate Driver"
    agent_container = "Agent Container"
    agent_discovery = "Agent Discovery"
    agent_discovery_base = "Agent Discovery Base"
    agent_driver = "Agent Driver"
    agent_driver_base = "Agent Driver Base"
    agent_module_handler = "Agent Module Handler"
    agent_module_handler_container = "Agent Module Handler Container"
    agent_root = "Agent Root"
    agent_ssh_base = "Agent SSH Base"
    agent_ssh_discovery = "Agent SSH Discovery"
    agent_ssh_driver = "Agent SSH Driver"
    amazon_app = "Amazon App"
    amazon_ca = "Amazon CA"
    amazon_credential = "Amazon Credential"
    apache = "Apache"
    apache_application_group = "Apache Application Group"
    aperture_configuration = "Aperture Configuration"
    application_base = "Application Base"
    application_group = "Application Group"
    associate_key_expiration_action = "Associate Key Expiration Action"
    auto_layout_manager = "Auto Layout Manager"
    automatic_password_credential = "Automatic Password Credential"
    azure_key_vault = "Azure Key Vault"
    basic = "Basic"
    bluecoat_sslva = "BlueCoat SSLVA"
    bluecoat_sslva_trust_store = "BlueCoat SSLVA Trust Store"
    branch_base = "Branch Base"
    bulk_application_base = "Bulk Application Base"
    bulk_provisioning_manager = "Bulk Provisioning Manager"
    ca_container = "CA Container"
    ca_import = "CA Import"
    ca_import_manager = "CA Import Manager"
    ca_root = "CA Root"
    capi = "CAPI"
    capi_trust_store = "CAPI Trust Store"
    csp_key_container = "CSP Key Container"
    certificate_authority_base = "Certificate Authority Base"
    certificate_credential = "Certificate Credential"
    certificate_manager = "Certificate Manager"
    certificate_pre_enrollment = "Certificate Pre-Enrollment"
    certificate_provisioning_work = "Certificate Provisioning Work"
    certificate_revocation = "Certificate Revocation"
    certificate_trust_bundle = "Certificate Trust Bundle"
    certificate_trust_store_base = "Certificate Trust Store Base"
    client_agent_automatic_upgrade_work = "Client Agent Automatic Upgrade Work"
    client_agent_configuration_work = "Client Agent Configuration Work"
    client_agent_device_placement_work = "Client Agent Device Placement Work"
    client_agent_ssh_discovery_work = "Client Agent SSH Discovery Work"
    client_agent_ssh_key_usage_work = "Client Agent SSH Key Usage Work"
    client_agent_ssh_provisioning_work = "Client Agent SSH Provisioning Work"
    client_certificate_discovery_work = "Client Certificate Discovery Work"
    client_certificate_work = "Client Certificate Work"
    client_group = "Client Group"
    client_group_base = "Client Group Base"
    client_group_container = "Client Group Container"
    client_group_root = "Client Group Root"
    client_portal_base = "Client Portal Base"
    client_root = "Client Root"
    client_subgroup = "Client Subgroup"
    client_user_certificate_work = "Client User Certificate Work"
    client_work_base = "Client Work Base"
    client_work_container = "Client Work Container"
    client_work_root = "Client Work Root"
    clientgroup_monitor = "ClientGroup Monitor"
    cloud_instance_monitor = "Cloud Instance Monitor"
    cloud_instance_monitor_driver_base = "Cloud Instance Monitor Driver Base"
    code_sign_manager = "Code Sign Manager"
    code_sign_preapproval_action = "Code Sign PreApproval Action"
    code_signing_action = "Code Signing Action"
    code_signing_admin_approval_action = "Code Signing Admin Approval Action"
    code_signing_apple_environment = "Code Signing Apple Environment"
    code_signing_apple_environment_template = "Code Signing Apple Environment Template"
    code_signing_application = "Code Signing Application"
    code_signing_application_collection = "Code Signing Application Collection"
    code_signing_application_container = "Code Signing Application Container"
    code_signing_application_root = "Code Signing Application Root"
    code_signing_csp_environment = "Code Signing CSP Environment"
    code_signing_csp_environment_template = "Code Signing CSP Environment Template"
    code_signing_certificate_environment = "Code Signing Certificate Environment"
    code_signing_certificate_environment_template = "Code Signing Certificate Environment Template"
    code_signing_delete_object_action = "Code Signing Delete Object Action"
    code_signing_dotnet_environment = "Code Signing DotNet Environment"
    code_signing_dotnet_environment_template = "Code Signing DotNet Environment Template"
    code_signing_environment_base = "Code Signing Environment Base"
    code_signing_environment_container = "Code Signing Environment Container"
    code_signing_environment_root = "Code Signing Environment Root"
    code_signing_environment_template_base = "Code Signing Environment Template Base"
    code_signing_environment_template_update_action = "Code Signing Environment Template Update Action"
    code_signing_flow_container = "Code Signing Flow Container"
    code_signing_flow_root = "Code Signing Flow Root"
    code_signing_gpg_environment = "Code Signing GPG Environment"
    code_signing_gpg_environment_template = "Code Signing GPG Environment Template"
    code_signing_key_pair_environment = "Code Signing Key Pair Environment"
    code_signing_key_pair_environment_template = "Code Signing Key Pair Environment Template"
    code_signing_key_time_constraint = "Code Signing Key Time Constraint"
    code_signing_object_management_action = "Code Signing Object Management Action"
    code_signing_pre_qualified_action = "Code Signing Pre Qualified Action"
    code_signing_project = "Code Signing Project"
    code_signing_project_container = "Code Signing Project Container"
    code_signing_project_root = "Code Signing Project Root"
    code_signing_root = "Code Signing Root"
    code_signing_time_constraint_root = "Code Signing Time Constraint Root"
    comodo_ca = "Comodo CA"
    comodo_ccm = "Comodo CCM"
    connectdirect = "ConnectDirect"
    connectdirect_trust_store = "ConnectDirect Trust Store"
    connection_base = "Connection Base"
    create_key_action = "Create Key Action"
    credential_base = "Credential Base"
    credential_container = "Credential Container"
    credential_driver_base = "Credential Driver Base"
    credential_root = "Credential Root"
    cyberark_credentials_driver = "CyberArk Credentials Driver"
    cyberark_password_credential = "CyberArk Password Credential"
    cyberark_username_password_credential = "CyberArk Username Password Credential"
    dpapi_encryption_driver = "DPAPI Encryption Driver"
    datapower = "DataPower"
    datapower_trust_store = "DataPower Trust Store"
    device = "Device"
    digicert_ca = "DigiCert CA"
    discovery = "Discovery"
    discovery_container = "Discovery Container"
    discovery_exclusion = "Discovery Exclusion"
    discovery_manager = "Discovery Manager"
    discovery_root = "Discovery Root"
    discovery_statistics = "Discovery Statistics"
    driver_base = "Driver Base"
    encryption_driver = "Encryption Driver"
    encryption_root = "Encryption Root"
    engine_root = "Engine Root"
    engine_upgrade = "Engine Upgrade"
    entrust_pki_gateway = "Entrust PKI Gateway"
    entrustnet_ca = "EntrustNET CA"
    exclusion = "Exclusion"
    f5_authentication_bundle = "F5 Authentication Bundle"
    f5_ltm_advanced = "F5 LTM Advanced"
    f5_ltm_advanced_trust_store = "F5 LTM Advanced Trust Store"
    flow = "Flow"
    flow_action_add_team_member = "Flow Action Add Team Member"
    flow_action_approver_base = "Flow Action Approver Base"
    flow_action_base = "Flow Action Base"
    flow_action_config_read_approvers = "Flow Action Config Read Approvers"
    flow_action_direct_approvers = "Flow Action Direct Approvers"
    flow_action_team_owner_approval = "Flow Action Team Owner Approval"
    flow_event_action = "Flow Event Action"
    flow_group = "Flow Group"
    flow_root = "Flow Root"
    gsk = "GSK"
    gsk_trust_store = "GSK Trust Store"
    generational_credential = "Generational Credential"
    generic_credential = "Generic Credential"
    globalsign_mssl_ca = "GlobalSign MSSL CA"
    google_cloud_app = "Google Cloud App"
    google_cloud_ca = "Google Cloud CA"
    google_credential = "Google Credential"
    group = "Group"
    http_ca_base = "HTTP CA Base"
    hashicorp_vault_pki = "HashiCorp Vault PKI"
    hydrantid_ca = "HydrantId CA"
    hydrantidv2_ca = "HydrantIdv2 CA"
    identity_driver = "Identity Driver"
    identity_root = "Identity Root"
    imperva_mx = "Imperva MX"
    integrated_credential_driver = "Integrated Credential Driver"
    intermediate_and_root_certificates = "Intermediate and Root Certificates"
    jks = "JKS"
    jks_trust_store = "JKS Trust Store"
    jss_cluster = "JSS Cluster"
    jss_container = "JSS Container"
    jss_discovery = "JSS Discovery"
    jss_discovery_manager = "JSS Discovery Manager"
    jss_namespace = "JSS Namespace"
    jump_server = "Jump Server"
    key_base = "Key Base"
    key_manager = "Key Manager"
    key_pair = "Key Pair"
    keynectis_sequoia_ca = "Keynectis Sequoia CA"
    ldap_identity_driver = "LDAP Identity Driver"
    layout_driver_base = "Layout Driver Base"
    layout_driver_container = "Layout Driver Container"
    layout_root = "Layout Root"
    layout_rule_base = "Layout Rule Base"
    layout_rule_container = "Layout Rule Container"
    legacy_key_base = "Legacy Key Base"
    license = "License"
    license_root = "License Root"
    log_adaptable = "Log Adaptable"
    log_application = "Log Application"
    log_application_container = "Log Application Container"
    log_application_customization = "Log Application Customization"
    log_channel = "Log Channel"
    log_channel_container = "Log Channel Container"
    log_file = "Log File"
    log_filter = "Log Filter"
    log_heartbeat = "Log Heartbeat"
    log_mssql = "Log MSSQL"
    log_notification = "Log Notification"
    log_notification_container = "Log Notification Container"
    log_root = "Log Root"
    log_smtp = "Log SMTP"
    log_snmp = "Log SNMP"
    log_sql_channel = "Log SQL Channel"
    log_server = "Log Server"
    log_syslog = "Log Syslog"
    metadata_base = "Metadata Base"
    metadata_category = "Metadata Category"
    metadata_choice = "Metadata Choice"
    metadata_container = "Metadata Container"
    metadata_datetime = "Metadata DateTime"
    metadata_identity = "Metadata Identity"
    metadata_list = "Metadata List"
    metadata_root = "Metadata Root"
    metadata_text = "Metadata Text"
    microsoft_ca = "Microsoft CA"
    microsoft_ca_pool = "Microsoft CA Pool"
    monitoring_base = "Monitoring Base"
    monitoring_module = "Monitoring Module"
    netscaler = "NetScaler"
    network_device_certificate_work = "Network Device Certificate Work"
    null_encryption_driver = "Null Encryption Driver"
    oauth_device_authorization_flow = "OAuth Device Authorization Flow"
    oauth_device_authorization_initialization_action = "OAuth Device Authorization Initialization Action"
    oauth_device_authorization_wait_for_identity_action = "OAuth Device Authorization Wait For Identity Action"
    onboard_discovery = "Onboard Discovery"
    onboard_discovery_manager = "Onboard Discovery Manager"
    openssl_ca = "OpenSSL CA"
    opentrust_pki_ca = "OpenTrust PKI CA"
    organization = "Organization"
    out_of_band_ca = "Out Of Band CA"
    pem = "PEM"
    pem_trust_store = "PEM Trust Store"
    pkcs_12 = "PKCS#12"
    pkcs_12_trust_store = "PKCS#12 Trust Store"
    palo_alto_network_fw = "Palo Alto Network FW"
    palo_alto_network_fw_trust_store = "Palo Alto Network FW Trust Store"
    password_credential = "Password Credential"
    person = "Person"
    pkcs11_encryption_driver = "Pkcs11 Encryption Driver"
    placement_job = "Placement Job"
    placement_job_container = "Placement Job Container"
    policy = "Policy"
    private_key_credential = "Private Key Credential"
    proxy = "Proxy"
    proxy_base = "Proxy Base"
    quovadis_ca = "QuoVadis CA"
    rsa_keon_ca = "RSA Keon CA"
    recycle_bin_owner_record = "Recycle Bin Owner Record"
    recycle_bin_root = "Recycle Bin Root"
    recycle_bin_vault_record = "Recycle Bin Vault Record"
    redhat_ca = "Redhat CA"
    remote_access_application = "Remote Access Application"
    remote_access_application_container = "Remote Access Application Container"
    remote_access_root = "Remote Access Root"
    remote_access_scope = "Remote Access Scope"
    remote_access_scope_container = "Remote Access Scope Container"
    report_base = "Report Base"
    report_container = "Report Container"
    report_filter_base = "Report Filter Base"
    report_root = "Report Root"
    report_template = "Report Template"
    report_analytics = "Report:Analytics"
    report_certificate_trends_report = "Report:Certificate Trends Report"
    report_certificateduplication = "Report:CertificateDuplication"
    report_certificateinventory = "Report:CertificateInventory"
    report_certificatenetworkvalidation = "Report:CertificateNetworkValidation"
    report_client_certificate_trends_report = "Report:Client Certificate Trends Report"
    report_clients = "Report:Clients"
    report_custom_report = "Report:Custom Report"
    report_demographics_certificate_authority = "Report:Demographics Certificate Authority"
    report_demographics_key_length = "Report:Demographics Key Length"
    report_demographics_signing_algorithm = "Report:Demographics Signing Algorithm"
    report_demographics_validity_period = "Report:Demographics Validity Period"
    report_demographics_wildcard = "Report:Demographics Wildcard"
    report_enhancedcertificateexpiration = "Report:EnhancedCertificateExpiration"
    report_entitlement = "Report:Entitlement"
    report_expiration = "Report:Expiration"
    report_historicalcertificates = "Report:HistoricalCertificates"
    report_licensing = "Report:Licensing"
    report_ssh_authorized_user = "Report:SSH Authorized User"
    report_ssh_discrepancy = "Report:SSH Discrepancy"
    report_ssh_key_expiration = "Report:SSH Key Expiration"
    report_ssh_server_summary = "Report:SSH Server Summary"
    report_ssh_trust = "Report:SSH Trust"
    report_ssh_user_key = "Report:SSH User Key"
    report_ssh_trends_report = "Report:Ssh Trends Report"
    report_trust = "Report:Trust"
    report_value = "Report:Value"
    reporter_service_module = "Reporter Service Module"
    riverbed_steelhead = "Riverbed SteelHead"
    saml = "SAML"
    saml_profile = "SAML Profile"
    ssh_automatic_rotation_trigger = "SSH Automatic Rotation Trigger"
    ssh_ca_key_pair = "SSH CA Key Pair"
    ssh_ca_key_pair_container = "SSH CA Key Pair Container"
    ssh_ca_template = "SSH CA Template"
    ssh_ca_template_container = "SSH CA Template Container"
    ssh_certificate_adaptable_flow_action = "SSH Certificate Adaptable Flow Action"
    ssh_certificate_base = "SSH Certificate Base"
    ssh_certificate_create_certificate_action = "SSH Certificate Create Certificate Action"
    ssh_certificate_create_public_key_action = "SSH Certificate Create Public Key Action"
    ssh_certificate_issuance_flow_container = "SSH Certificate Issuance Flow Container"
    ssh_certificate_manager = "SSH Certificate Manager"
    ssh_certificate_start_processing_action = "SSH Certificate Start Processing Action"
    ssh_client_certificate = "SSH Client Certificate"
    ssh_host_certificate = "SSH Host Certificate"
    ssh_key = "SSH Key"
    ssh_manager = "SSH Manager"
    ssh_manual_install_key_device = "SSH Manual Install Key Device"
    ssh_server_key = "SSH Server Key"
    ssh_user_key = "SSH User Key"
    ssh_work = "SSH Work"
    sso = "SSO"
    schedule_base = "Schedule Base"
    secret_key = "Secret Key"
    secret_store_root = "Secret Store Root"
    self_signed_ca = "Self Signed CA"
    server_agent_base_device_placement_work = "Server Agent Base Device Placement Work"
    server_agent_cert_device_placement_work = "Server Agent Cert Device Placement Work"
    server_agent_ssh_device_placement_work = "Server Agent SSH Device Placement Work"
    server_certificate_work = "Server Certificate Work"
    service_module = "Service Module"
    spi_logger = "Spi Logger"
    ssh_device_base = "Ssh Device Base"
    statistics_container = "Statistics Container"
    statistics_counter = "Statistics Counter"
    statistics_root = "Statistics Root"
    symmetric_key = "Symmetric Key"
    tealeaf_pca = "Tealeaf PCA"
    tealeafsunimp = "TealeafSunimp"
    top = "Top"
    tree_root = "Tree Root"
    trust = "Trust"
    trustwave_ca = "Trustwave CA"
    untrusted_client_group = "Untrusted Client Group"
    upgrade = "Upgrade"
    upgrade_task = "Upgrade Task"
    upgrades_root = "Upgrades Root"
    user = "User"
    user_preference = "User Preference"
    username_password_credential = "Username Password Credential"
    validation_base = "Validation Base"
    validation_manager = "Validation Manager"
    venafi_platform = "Venafi Platform"
    workflow = "Workflow"
    workflow_container = "Workflow Container"
    workflow_root = "Workflow Root"
    workflow_ticket = "Workflow Ticket"
    x509_certificate = "X509 Certificate"
    x509_certificate_base = "X509 Certificate Base"
    x509_certificate_validation = "X509 Certificate Validation"
    x509_code_signing_certificate = "X509 Code Signing Certificate"
    x509_device_certificate = "X509 Device Certificate"
    x509_intermediate_root_certificate = "X509 Intermediate Root Certificate"
    x509_intermediate_root_certificate_container = "X509 Intermediate Root Certificate Container"
    x509_root_certificate = "X509 Root Certificate"
    x509_root_certificate_container = "X509 Root Certificate Container"
    x509_server_certificate = "X509 Server Certificate"
    x509_time_stamping_certificate = "X509 Time Stamping Certificate"
    x509_user_certificate = "X509 User Certificate"
    zone_base = "Zone Base"
    iplanet = "iPlanet"
