from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class AmazonCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Amazon CA"
    access_key_id = Attribute('Access Key ID', min_version='21.4')
    amazon_private_ca = Attribute('Amazon Private CA', min_version='21.4')
    aws_credential_dn = Attribute('Aws Credential DN', min_version='21.4')
    domain_control_validation = Attribute('Domain Control Validation', min_version='21.4')
    interval = Attribute('Interval', min_version='21.4')
    keys_generated_by_the_ca = Attribute('Keys Generated By The CA', min_version='21.4')
    region_code = Attribute('Region Code', min_version='21.4')
    retrieval_period = Attribute('Retrieval Period', min_version='21.4')
    secret_access_key = Attribute('Secret Access Key', min_version='21.4')
    signature_algorithm = Attribute('Signature Algorithm', min_version='21.4')
    use_private_ca = Attribute('Use Private CA', min_version='21.4')
